function data = cocktail_inv(len,input)
% Computes inverse CDF, used to generate random samples from the cocktail
% distribution.
% len: Integer number of samples desired, eg, 1000.
% "Input" denotes a structure with the
% following fields: rho_PL, rho_FLN, rho_BLN, alpha, omega_LN, omega_PL,and sigma (please see the
% Appendix of the paper "The self-organization of a spoken word (2012), (Holden and
% Rajaraman." for more details).
%
% If the 5 free paramters rho_PL, alpha, omega_LN, omega_PL, and sigma are
% input, then rho_FLN & rho_BLN will be computed. This input method insures
% a smooth PDF as output.
% Release 1.1
% edited by JH 9/24/2011 - changed input to integer, rnd generated in file,
% col vector output.
% edited by JH 5/24/2013 - introduced test that allowes 5 parameter input.

field_names = {'rho_PL','rho_FLN','rho_BLN','alpha','omega_LN','omega_PL','sigma'};
ind = find(~isfield(input,field_names));

if isempty(ind)
   
elseif (ind(1)==2 && ind(2)==3 && size(ind,2)==2)
        % compute rho_FLN & rho_BLN from 5 input parameters
        % Insures continuity.
        A = lognpdf(input.omega_PL,input.omega_LN,input.sigma);
        C = logncdf(input.omega_PL,input.omega_LN,input.sigma);
        psi = (input.alpha-1)/input.omega_PL/A;
        input.rho_FLN =  C*(1-input.rho_PL) + C*(1-C)*input.rho_PL*psi;
        input.rho_BLN = 1 - (input.rho_FLN + input.rho_PL);

elseif ~isempty(ind)
    str = []; for i=1:length(ind), str = [str,' ',field_names{ind(i)}]; end
    error(['The following fields:',str,', are missing.']);
end

rnd=rand(1,len); % relies on defult random seed settings
mag = [exp(input.omega_LN-input.sigma^2),input.omega_PL];

[~,inx] = max(cocktail_pdf(mag,input));

data0 = mag(inx)*ones(size(rnd));

Reltol = 1e-6;

% Newton-Raphson method for inverting CDF;

while 1
    
C = cocktail_cdf(data0,input) - rnd;

B = cocktail_pdf(data0,input) + eps;  % adding tolerance to prevent ill-conditioning


data = data0 - C./B;

if norm((data-data0))/norm(data) < Reltol
    
    break;
    
else 
    
    data0 = data;
    
end

end
    %data=round(data); % outputs a row vector
    data=round(data(:)); % outputs a column vector