function output = cocktail_model_estimation(data)
%function output = cocktail_model_estimation(data)
% This is the main script that estimates the parameters of the cocktail
% model for a set of data.  This code computes the parameters, using the
% method of maximum likelihood estimation (MLE), although other methods of
% parameter estimation can be readily applied.
%
% Here, output is a structure variable that contains the following fields:
% 1) rho_FLN: the proportion of the lognormal distribution in the front end
% 2) rho_BLN: the proportion of the lognormal distribution in the back end
% 3) rho_PL: the proportion of the inverse power law distribution (exclusively in the back end)
% 4) omega_LN: the mean of the distribution of the natural logarithm of the random variable
% 5) sigma: the standard deviation of the distribution of the natural logarithm of the random variable
% 6) alpha:  the scaling exponent of the inverse power law distribution
% 7) omega_PL: the threshold parameter of the inverse power law distribution
% More details can be obtained from the article: The self-organization of a spoken word (2012), (Holden and
% Rajaraman.
%
% data: is a vector of data points whose distribution is being estimated.
% All enties in the vector data less than zero are automatically purged
% from the estimation procedure
%
% Author: Srinivasan Rajaraman
% E-mail: srinivasan.rajaraman@gmail.com
% Release: 1
% Release date: July 22, 2012

%% Initalize paramters and options

data(data <0) = [];  % First purge all the data points that are less than or equal to zero

% Set options structure, used with fminsearch
opts = optimset('fminsearch'); opts.TolX = 1e-8; opts.TolFun = 1e-8; opts.MaxIter = 5e+3; opts.MaxFunEvals = 5e+3; opts.Display = 'on';

% Initialize the parameter structure
optN = sshist(data,[2:length(data)-1]);
[N,X] = hist(data,optN);
[~,id] = max(N); X_max = X(id);
param_init.omega_LN = median(log(data(data <= X_max)));
param_init.sigma = std(log(data(data <= X_max)));
param_init.alpha = 5;
param_init.omega_PL = exp(param_init.omega_LN + param_init.sigma^2*(param_init.alpha-1));
A = lognpdf(param_init.omega_PL,param_init.omega_LN,param_init.sigma);
C = logncdf(param_init.omega_PL,param_init.omega_LN,param_init.sigma);
psi = (param_init.alpha-1)/(param_init.omega_PL*A);
param_init.rho_PL  = 0.5/(1+C*psi);

%% Maximum likelihood estimation
LL_Function = @(x) 2*sum(log(cocktail_pdf(data,xtransform(x))));  % The log-likelihood function

%standard matlab fminsearch
%sols = fminsearch(@(x) -LL_Function(x),invxtransform(param_init),opts); % Finding the minima of the negative loglikelihood function

%modified version of fminsearch that seems to work better.
sols = fmins_new(@(x) -LL_Function(x),invxtransform(param_init),1e-8,0); % Finding the minima of the negative loglikelihood function

output = xtransform(sols);  % The structure of the maximum likelihood estimation parameters 


%% A subroutine to convert the deviation variables to the parameter structure
function param = xtransform(xin)

param.omega_LN = xin(1);
param.sigma = 10^xin(2);
param.alpha = 1 + 10^xin(3);
param.omega_PL = exp(param.omega_LN + param.sigma^2*(param.alpha-1));
C = logncdf(param.omega_PL,param.omega_LN,param.sigma);
A = lognpdf(param.omega_PL,param.omega_LN,param.sigma);
psi = (param.alpha-1)/(param.omega_PL*A);
param.rho_PL = (10^xin(4)/(1+10^xin(4)))/(1+C*psi);
param.rho_FLN = C*(1-param.rho_PL) + (C*(1-C)*param.rho_PL*(param.alpha-1))/(param.omega_PL*A);
param.rho_BLN = 1 - (param.rho_PL + param.rho_FLN);


%% A subroutine to convert the parameter structure into deviation variables
function xout = invxtransform(param)
xout(1) = param.omega_LN;
xout(2) = log10(param.sigma);
xout(3) = log10(param.alpha-1);        
thres = exp(param.omega_LN + param.sigma^2*(param.alpha-1)); 
psi = (param.alpha-1)/param.omega_PL/lognpdf(param.omega_PL,param.omega_LN,param.sigma);
C = logncdf(param.omega_PL,param.omega_LN,param.sigma);
xout(4) = log10(param.rho_PL/(1/(1+C*psi)-param.rho_PL));

