% function output = cocktail_pdf(data,input)
% This script computes the cocktail model PDF for a vector of data points
% contained in the variable "data." "Input" denotes a structure with the
% following fields: rho_PL, rho_FLN, rho_BLN, alpha, omega_LN, omega_PL,and sigma (please see the
% Appendix of the paper "The self-organization of a spoken word (2012), (Holden and
% Rajaraman." for more details)
% If the 5 free paramters rho_PL, alpha, omega_LN, omega_PL,and sigma are
% input, then rho_FLN & rho_BLN will be computed. This input method insures
% a smooth PDF as output.
%
% output: is the PDF values for the vector of data points given in the
% variable "data."
%
% Author: Srinivasan Rajaraman
% E-mail: srinivasan.rajaraman@gmail.com
% Release: 1.1
% Release date: July 22, 2012
% Edited by Jay Holden, May 24, 2013
function output = cocktail_pdf(data,input)
field_names = {'rho_PL','rho_FLN','rho_BLN','alpha','omega_LN','omega_PL','sigma'};
ind = find(~isfield(input,field_names));

if isempty(ind)
   
elseif (ind(1)==2 && ind(2)==3 && size(ind,2)==2)
        % compute rho_FLN & rho_BLN from 5 input parameters
        % Insures continuity.
        A = lognpdf(input.omega_PL,input.omega_LN,input.sigma);
        C = logncdf(input.omega_PL,input.omega_LN,input.sigma);
        psi = (input.alpha-1)/input.omega_PL/A;
        input.rho_FLN =  C*(1-input.rho_PL) + C*(1-C)*input.rho_PL*psi;
        input.rho_BLN = 1 - (input.rho_FLN + input.rho_PL);

elseif ~isempty(ind)
    str = []; for i=1:length(ind), str = [str,' ',field_names{ind(i)}]; end
    error(['The following fields:',str,', are missing.']);
end

C = logncdf(input.omega_PL,input.omega_LN,input.sigma);
I_log = find(data < input.omega_PL);
output(I_log) = (input.rho_FLN/C)*lognpdf(data(I_log),input.omega_LN,input.sigma);
I_log_pl = find(data >= input.omega_PL);
output(I_log_pl) = (input.rho_BLN/(1-C))*lognpdf(data(I_log_pl),input.omega_LN,input.sigma) + input.rho_PL*...
                   (input.alpha-1)*(input.omega_PL)^(input.alpha-1)*data(I_log_pl).^(-input.alpha);


